import logging
import random
import string
import xml.etree.ElementTree as etree
from typing import Literal

from markdown import Markdown
from pymdownx.blocks import BlocksExtension, BlocksProcessor  # type: ignore
from pymdownx.blocks.block import Block, type_string_in  # type: ignore

log = logging.getLogger(f"mkdocs.extensions.{__name__}")


def generate_random_string(size: int) -> str:
    return "".join(
        random.choices(string.ascii_letters + string.digits, k=size)
    )


class EchartsBlock(Block):
    NAME = "echarts"
    ARGUMENT = False
    OPTIONS = {
        "renderer": ("svg", type_string_in(["svg", "canvas"])),
    }

    md: Markdown

    # custom state
    echarts_options: str
    raw_content: str
    div_id: str

    def on_init(self):
        self.echarts_options = ""
        self.div_id = ""
        self.raw_content = ""

    @property
    def attrs(self) -> dict:
        return self.options.get("attrs", {})

    @property
    def renderer(self) -> Literal["svg", "canvas"]:
        return self.options.get("renderer", "svg")

    def on_create(self, parent: etree.Element):  # type: ignore
        """Called when a block is initially found and initialized.
        The on_create method should create the container for the
        block under the parent element. Other child elements can
        be created on the root of the container, but outer element
        of the created container should be returned.
        """

        # generate a random id if not provided
        log.debug("echarts block found")
        self.div_id = self.attrs.get("id", generate_random_string(6))
        div = etree.SubElement(parent, "div")
        return div

    def on_add(self, block: etree.Element):
        """When any calls occur to process new content, on_add is called.
        This gives the block a chance to return the element where the
        content is desired."""

        # save the raw content in all cases
        self.raw_content = block.text or ""
        # ignore empty block
        if block.text is None:
            return block
        # check if it looks like a json object
        start = block.text.find("{")
        end = block.text.rfind("}")
        if start == -1 or end == -1:
            return block

        log.debug("echarts options can be extracted from the block")
        self.echarts_options = block.text[start : end + 1]
        # remove the text (otherwise the raw text will be displayed)
        block.text = ""
        return block

    def on_end(self, block: etree.Element) -> None:
        """When a block is parsed to completion, the on_end event is
        executed. This allows an extension to perform any post
        processing on the elements. You could save the data as raw
        text and then parse it special at the end or you could walk
        the HTML elements and move content around, add attributes, or
        whatever else is needed."""
        log.debug(f"content of echarts block:\n{self.raw_content}")
        if self.echarts_options == "":
            log.warning(
                f"No echarts options found in the following block:"
                f"\n---\n{self.raw_content}\n---\n"
            )
            return

        script = etree.SubElement(block, "script")
        script.set("type", "text/javascript")
        script.set("defer", "true")

        script.text = f"""
        const target{self.div_id} = document.getElementById('{self.div_id}');
        const chart{self.div_id} = echarts.init(target{self.div_id}, 'shadcn', {{ renderer: '{self.renderer}' }});
        chart{self.div_id}.setOption({self.echarts_options});
        window.addEventListener('resize', function() {{ chart{self.div_id}.resize(); }});
        const observer{self.div_id} = new IntersectionObserver((entries) => {{
            entries.forEach(entry => {{
                if (entry.isIntersecting) {{
                    chart{self.div_id}.resize();
                    observer{self.div_id}.disconnect(); // Stop observing if you want a one-time trigger
                }}
            }});
        }});

        observer{self.div_id}.observe(target{self.div_id});
        """

        # set some attributes if not defined by the user
        block.set("id", self.div_id)
        block.set("class", block.get("class", "echarts"))
        block.set("style", block.get("style", "width:100%;height:500px;"))

        self.echarts_options = ""
        self.div_id = ""

    def on_markdown(self) -> str:  # type: ignore
        """Check how element should be treated by the Markdown parser."""
        return "raw"


class EchartsBlockExtension(BlocksExtension):
    def extendMarkdownBlocks(self, md: Markdown, block_mgr: BlocksProcessor):
        block_mgr.register(EchartsBlock, self.getConfigs())


def makeExtension(*args, **kwargs):
    """Return extension."""
    return EchartsBlockExtension(*args, **kwargs)



import json
import os

from bottle import request, response  # type: ignore
from mkdocs.config import config_options as c
from mkdocs.config.base import Config
from mkdocs.config.defaults import MkDocsConfig
from mkdocs.plugins import BasePlugin, get_plugin_logger

from ._router import RouterMixin

log = get_plugin_logger("excalidraw")


def svg_handler_factory(directory: str):
    """
    Attributes:
        directory (str): Directory where the SVG files are stored. This is relative to the docs_dir.
    """

    def handler():
        file = request.query.get("file")
        svg_file = os.path.join(directory, file.replace(".json", ".svg"))

        if request.method == "POST":
            log.info(f"POST {request.path}?{request.query_string}")
            log.debug(f"opening {svg_file} for writing request body")
            with open(svg_file, "wb") as f:
                f.write(request.body.read())

        elif request.method == "GET":
            log.info(f"GET {request.path}?{request.query_string}")
            if not os.path.exists(svg_file):
                log.warning(f"file {svg_file} not found, creating it")
                with open(svg_file, "w") as f:
                    # default height to 400px
                    f.write(
                        f"""<svg id="{file}" version="1.1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 670 400" width="100%" height="400"></svg>"""
                    )
            response.content_type = "image/svg+xml"
            return open(svg_file, "r").read().strip()

    return handler


def scene_handler_factory(directory: str):
    def handler():
        file = request.query.get("file")
        scene_file = os.path.join(directory, file)

        if request.method == "POST":
            log.info(f"POST {request.path}?{request.query_string}")
            data = request.json
            if data:
                # we have an issue since excalidraw pass an object but
                # expect an array
                log.debug(f"saving state to {scene_file}")
                del data["appState"]["collaborators"]
                with open(scene_file, "w") as f:
                    json.dump(data, f, indent=2)
            else:
                log.warning("no data in request body")

        elif request.method == "GET":
            log.info(f"GET {request.path}?{request.query_string}")
            if not os.path.exists(scene_file):
                log.warning(f"file {scene_file} not found, creating it")
                with open(scene_file, "w") as f:
                    json.dump({"elements": [], "appState": {}, "files": {}}, f)

            response.content_type = "application/json"
            return open(scene_file, "r").read().strip()

    return handler


class ExcalidrawPluginConfig(Config):
    # Directory where the excalidraw files are stored
    # relative to the docs_dir
    directory = c.Dir(
        exists=False,
        default="excalidraw",
    )


class ExcalidrawPlugin(RouterMixin, BasePlugin[ExcalidrawPluginConfig]):
    """This plugin enabled the real time edition of
    excalidraw scenes in development mode"""

    is_dev_server = False
    """Internal flag to detect if we are in development mode."""

    def on_startup(self, *, command, dirty: bool):
        """Detect if the server is running in development mode."""
        self.is_dev_server = command == "serve"

    def on_config(self, config: MkDocsConfig, **kwargs):
        """Three operations are performed:

        - detect and create the excalidraw directory
        - load the internal excalidraw markdown extension
        - inject the HTTP routes needed to handle excalidraw scenes and SVGs
        """
        base = os.path.dirname(config["config_file_path"])
        excalidraw_path = os.path.join(base, self.config.directory)
        extension_config = {
            "base_dir": excalidraw_path,
            "svg_only": not self.is_dev_server,
        }
        log.debug(
            f"Loading markdown extension 'shadcn.extensions.excalidraw' "
            f"with configuration: {extension_config}"
        )
        config["markdown_extensions"].append("shadcn.extensions.excalidraw")
        config["mdx_configs"]["shadcn.extensions.excalidraw"] = (
            extension_config
        )
        # create directory
        log.debug(f"creating excalidraw directory: {excalidraw_path}")
        os.makedirs(excalidraw_path, exist_ok=True)

        # these routes are injected in on_serve
        log.debug("injecting HTTP routes for excalidraw plugin")
        self.add_route(
            "/excalidraw/scene",
            scene_handler_factory(excalidraw_path),
            method=["GET", "POST"],
        )
        self.add_route(
            "/excalidraw/svg",
            svg_handler_factory(excalidraw_path),
            method=["GET", "POST"],
        )



from typing import Union

from git import Repo
from mkdocs.config.defaults import MkDocsConfig
from mkdocs.plugins import get_plugin_logger
from mkdocs.structure.files import Files
from mkdocs.structure.pages import Page

from shadcn.plugins.mixins.base import Mixin

REPO_CONFIG_KEY = "git_repository"
CREATED_AT_META_KEY = "created_at"
UPDATED_AT_META_KEY = "updated_at"


logger = get_plugin_logger("mixins/git")


def find_repo(abs_src_file: str) -> Union[Repo, None]:
    """
    Find the git repository for the given source file.
    Returns None if no repository is found.
    """
    try:
        return Repo(abs_src_file, search_parent_directories=True)
    except Exception:
        print(f"Could not find git repository starting from {abs_src_file}")
        return None


class GitTimestampsMixin(Mixin):
    def on_config(self, config: MkDocsConfig):
        """Called when the config is loaded.

        Attributes:
            config (dict): The MkDocs configuration dictionary.

        """
        config[REPO_CONFIG_KEY] = find_repo(config.config_file_path)
        logger.debug(f"Git repository: {config[REPO_CONFIG_KEY]}")
        return super().on_config(config)

    def on_page_markdown(
        self,
        markdown: str,
        /,
        *,
        page: Page,
        config: MkDocsConfig,
        files: Files,
    ):
        repo = config.get(REPO_CONFIG_KEY, None)
        if isinstance(repo, Repo) and page.file.abs_src_path:
            dates = [
                commit.committed_datetime
                for commit in repo.iter_commits(paths=page.file.abs_src_path)
            ]
            if len(dates) > 0:
                page.meta[CREATED_AT_META_KEY] = dates[-1]
                page.meta[UPDATED_AT_META_KEY] = dates[0]

        return super().on_page_markdown(
            markdown, page=page, config=config, files=files
        )



from jinja2 import Environment
from mkdocs.config.defaults import MkDocsConfig
from mkdocs.plugins import get_plugin_logger
from mkdocs.structure.files import Files

from shadcn.plugins.mixins.base import Mixin

logger = get_plugin_logger("mixins/dev")


class DevServerMixin(Mixin):
    """A mixin to add development server capabilities to MkDocs plugins."""

    def on_startup(self, *, command, dirty):
        self.is_dev_server = command == "serve"
        logger.debug(f"Dev server: {self.is_dev_server}")
        super().on_startup(command=command, dirty=dirty)

    def on_env(
        self, env: Environment, /, *, config: MkDocsConfig, files: Files
    ) -> Environment:
        env.globals["is_dev_server"] = self.is_dev_server
        return super().on_env(env, config=config, files=files)

    def on_config(self, config: MkDocsConfig):
        # dev server detection
        config["is_dev_server"] = self.is_dev_server
        return super().on_config(config)



import urllib.parse
import urllib.request
from functools import lru_cache
from pathlib import Path
from typing import Any, Union

from mkdocs.config.defaults import MkDocsConfig
from mkdocs.structure.nav import Navigation, Section
from mkdocs.structure.pages import Page


@lru_cache()
def iconify(key: str, height: str = "20px", **kwargs) -> str:
    base_url = "https://api.iconify.design"
    icon = key.split(":")
    if len(icon) != 2:
        raise ValueError(
            f"Invalid icon format: {key}. Expected format 'provider:name'."
        )
    # collapse icon
    provider, name = icon
    url = f"{base_url}/{provider}/{name}.svg?{urllib.parse.urlencode({'height': height, **kwargs})}"
    with urllib.request.urlopen(url) as response:
        content = response.read().decode(
            "utf-8"
        )  # Convert to string if needed
    return content


def parse_author(site_author: str) -> Union[str, None]:
    """Returns the email address of the site author."""
    # parse thinks like "Alban Siffer <31479857+asiffer@users.noreply.github.com>"
    if "<" in site_author and ">" in site_author:
        chunks = site_author.split("<")
        email = chunks[-1].split(">")[0]
        name = chunks[0].strip()
    else:
        email = None
        name = site_author.strip()

    if email:
        return f'<a href="mailto:{email}">{name}</a>'
    return f"<span>{name}</span>"


def setattribute(value: Union[dict, object], k: str, v: Any):
    if hasattr(value, "__setattr__"):
        setattr(value, k, v)
    return value


def active_section(nav: Navigation) -> Union[Section, None]:
    """Return the top-level active section"""
    for item in nav:
        if isinstance(item, Section) and item.is_section and item.active:
            return item
    return None


def first_page(section: Section) -> Union[Page, None]:
    """Return the first page in a section"""
    for item in section.children:
        if isinstance(item, Page) and item.is_page:
            return item

    for item in section.children:
        if isinstance(item, Section):
            fp = first_page(item)
            if fp:
                return fp

    return None


def file_exists(path: str, config: MkDocsConfig) -> bool:
    """Check if a file exists at the given path, from docs_dir"""
    p: Path = Path(config.docs_dir) / Path(path)
    return p.exists() and p.is_file()


def is_http_url(path: str) -> bool:
    """Check if a path is a valid URL (http, https and also data scheme)"""
    try:
        parsed = urllib.parse.urlparse(path)
    except Exception:
        return False

    if parsed.scheme not in ("http", "https", "data"):
        return False
    return True



from urllib.parse import urljoin
from mkdocs.config.defaults import MkDocsConfig
from mkdocs.plugins import get_plugin_logger
from mkdocs.utils.templates import TemplateContext
from mkdocs.structure.pages import Page
from shadcn.plugins.mixins.base import Mixin
import os
import shutil

logger = get_plugin_logger("mixins/markdown")


class MarkdownMixin(Mixin):
    """A mixin to expose raw page markdown in templates, copy them to the build dir, and provide a URL."""

    def __init__(self):
        self.raw_markdown = {}

    def on_page_context(
        self, context: TemplateContext, page: Page, config: MkDocsConfig, nav
    ):
        self.raw_markdown[page.file.abs_src_path] = os.path.join(
            config.site_dir, page.file.src_path
        )
        context.update(
            {
                "raw_markdown_url": urljoin(
                    config.site_url or "/", page.file.src_path
                )
            }
        )
        return context

    def on_post_build(self, config):
        # Copy raw markdown files to the build directory
        for src, dst in self.raw_markdown.items():
            logger.debug(f"Copying raw markdown file {src} to {dst}")
            shutil.copy2(src, dst)
        return super().on_post_build(config)



import logging
import random
import re
import string
import xml.etree.ElementTree as etree
from typing import Literal

from markdown import Markdown
from pymdownx.blocks import BlocksExtension, BlocksProcessor  # type: ignore
from pymdownx.blocks.block import Block, type_string_in  # type: ignore

log = logging.getLogger(f"mkdocs.extensions.{__name__}")


def generate_random_string(size: int) -> str:
    return "".join(
        random.choices(string.ascii_letters + string.digits, k=size)
    )


class CodexecBlock(Block):
    NAME = "codexec"
    ARGUMENT = False
    OPTIONS = {
        "renderer": ("svg", type_string_in(["svg", "canvas"])),
    }

    md: Markdown

    def on_init(self):
        self.codexec_options = ""
        self.div_id = ""
        self.raw_content = ""

    @property
    def attrs(self) -> dict:
        return self.options.get("attrs", {})

    @property
    def renderer(self) -> Literal["svg", "canvas"]:
        return self.options.get("renderer", "svg")

    def on_create(self, parent: etree.Element):  # type: ignore
        """Called when a block is initially found and initialized.
        The on_create method should create the container for the
        block under the parent element. Other child elements can
        be created on the root of the container, but outer element
        of the created container should be returned.
        """
        div = etree.SubElement(parent, "div")
        div.set("class", "codexec")
        return div

    def on_end(self, block: etree.Element) -> None:
        """When a block is parsed to completion, the on_end event is
        executed. This allows an extension to perform any post
        processing on the elements. You could save the data as raw
        text and then parse it special at the end or you could walk
        the HTML elements and move content around, add attributes, or
        whatever else is needed."""

        code = block.find(".//code")
        if code is None:
            log.warning(
                "CodexecBlock: No <code> element found in block. "
                "This block will not be rendered."
            )
            return

        regex = re.compile(r"^:::([a-zA-Z0-9_]+)\s*")
        content = (code.text or "").strip(" \n\t")
        m = regex.match(content)
        if m:
            # Extract the language from the first line
            self.lang = m.group(1)
            content = content[m.end() :].strip(" \n\t")

        content = re.sub(
            r"(?<!\\)\\(.)",
            r"\\\\\1",
            content,
        )  # Escape single backslashes

        # run button
        button = etree.SubElement(block, "button")
        callback = """({output, ok}) => { 
            const result = this.parentElement.getElementsByClassName('codexec-result')[0]; 
            if (result) { 
                if (ok === false) {
                    result.classList.add('error');
                } else {
                    result.classList.remove('error');
                }
                const pre = result.querySelector('pre');
                if (pre) {
                    pre.innerHTML = output;
                }
            }
            this.dataset.status = '';
        }
        """
        button.set("data-status", "")
        button.set(
            "onclick",
            f"""this.dataset.status='loading';runCodexec(`{content}`,'{self.lang}').then({callback})""",
        )
        icon = etree.SubElement(button, "svg")
        icon.set("class", "play")
        icon.set("xmlns", "http://www.w3.org/2000/svg")
        icon.set("viewBox", "0 0 24 24")
        icon.set("fill", "currentColor")
        path = etree.SubElement(icon, "path")
        path.set("fill-rule", "evenodd")
        path.set(
            "d",
            "M4.5 5.653c0-1.427 1.529-2.33 2.779-1.643l11.54 6.347c1.295.712 1.295 2.573 0 3.286L7.28 19.99c-1.25.687-2.779-.217-2.779-1.643V5.653Z",
        )

        # <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="lucide lucide-loader-circle-icon lucide-loader-circle"><path d="M21 12a9 9 0 1 1-6.219-8.56"/></svg>
        icon = etree.SubElement(button, "svg")
        icon.set("class", "loading animate-spin")
        icon.set("xmlns", "http://www.w3.org/2000/svg")
        icon.set("viewBox", "0 0 24 24")
        icon.set("stroke", "currentColor")
        icon.set("stroke-width", "2")
        icon.set("stroke-linecap", "round")
        icon.set("stroke-linejoin", "round")
        icon.set("fill", "none")
        path = etree.SubElement(icon, "path")
        path.set(
            "d",
            "M21 12a9 9 0 1 1-6.219-8.56",
        )

        # add result container
        result = etree.SubElement(block, "div")
        result.set("class", "codexec-result")
        etree.SubElement(result, "pre")

    def on_markdown(self) -> str:  # type: ignore
        #     """Check how element should be treated by the Markdown parser."""
        return "block"


class CodexecBlockExtension(BlocksExtension):
    def extendMarkdownBlocks(self, md: Markdown, block_mgr: BlocksProcessor):
        block_mgr.register(CodexecBlock, self.getConfigs())


def makeExtension(*args, **kwargs):
    """Return extension."""
    return CodexecBlockExtension(*args, **kwargs)



from typing import Any

from jinja2 import Environment
from mkdocs.config.defaults import MkDocsConfig
from mkdocs.plugins import get_plugin_logger
from mkdocs.structure.files import Files
from mkdocs.structure.nav import Navigation
from mkdocs.structure.pages import Page

mixin_logger = get_plugin_logger("mixins")


class Mixin:
    """A base mixin class for MkDocs plugins."""

    def _super_method_or(
        self,
        method_name: str,
        *args,
        fallback: Any = None,
        **kwargs,
    ):
        """Call the superclass method if it exists, otherwise return the fallback value."""
        fun = getattr(super(), method_name, None)
        if fun is None:
            return fallback
        return fun(*args, **kwargs)

    def on_startup(self, *, command, dirty) -> None:
        return self._super_method_or(
            "on_startup",
            command=command,
            dirty=dirty,
        )

    def on_env(
        self,
        env: Environment,
        /,
        *,
        config: MkDocsConfig,
        files: Files,
    ) -> Environment:
        return self._super_method_or(
            "on_env",
            env,
            config=config,
            files=files,
            fallback=env,
        )

    def on_nav(
        self,
        nav: Navigation,
        /,
        *,
        config: MkDocsConfig,
        files: Files,
    ) -> Navigation:
        return self._super_method_or(
            "on_nav",
            nav,
            config=config,
            files=files,
            fallback=nav,
        )

    def on_page_markdown(
        self,
        markdown: str,
        /,
        *,
        page: Page,
        config: MkDocsConfig,
        files: Files,
    ) -> str:
        return self._super_method_or(
            "on_page_markdown",
            markdown,
            page=page,
            config=config,
            files=files,
            fallback=markdown,
        )

    def on_config(self, config: MkDocsConfig):
        return self._super_method_or("on_config", config, fallback=config)



import re
from typing import List, Set

from mkdocs.config.defaults import MkDocsConfig
from mkdocs.plugins import get_plugin_logger
from mkdocs.structure.files import Files
from mkdocs.structure.nav import Navigation, Section
from mkdocs.structure.pages import Page

from shadcn.plugins.mixins.base import Mixin

ORDER_META_KEY = "order"

logger = get_plugin_logger("mixins/order")


class OrderMixin(Mixin):
    page_index = 0
    """Internal page index for orderning purpose"""
    page_indices: Set[int] = set()
    """Internal set of pages that have hard-coded order"""

    nav_order: List[str] = []
    """Internal list of pages in the order they appear in the navigation"""

    def pre_order(self, nav_items: list):
        for item in nav_items:
            if isinstance(item, Page):
                self.nav_order.append(item.file.src_path)
            elif isinstance(item, Section):
                self.pre_order(item.children)

    def on_nav(
        self,
        nav: Navigation,
        /,
        *,
        config: MkDocsConfig,
        files: Files,
    ) -> Navigation:
        # if we create folders with 00_name_of_the_folder we remove the prepended number
        # from the title. It is a common hack to have the folders ordered in the navigation
        rex = re.compile(r"^[0-9]+[ _]")
        for item in nav.items:
            if isinstance(item, Section) and rex.match(item.title):
                item.title = rex.sub("", item.title).capitalize()

        # save the nav order for later use
        self.pre_order(nav.items)
        logger.debug(f"Navigation order: {self.nav_order}")
        return super().on_nav(nav, config=config, files=files) or nav

    def on_page_markdown(
        self,
        markdown: str,
        /,
        *,
        page: Page,
        config: MkDocsConfig,
        files: Files,
    ) -> str:
        # add order to page if not defined
        # It uses the following priority:
        # 1. order defined in the page's meta (frontmatter)
        # 2. order defined by the navigation position
        # 3. order defined by the order of processing pages (alphabetically)
        try:
            page.meta[ORDER_META_KEY] = page.meta.get(
                ORDER_META_KEY,
                self.nav_order.index(page.file.src_path),
            )
        except ValueError:
            page.meta[ORDER_META_KEY] = self.page_index

        self.page_indices.add(self.page_index)
        # increment page index
        while self.page_index in self.page_indices:
            self.page_index += 1

        return super().on_page_markdown(
            markdown,
            page=page,
            config=config,
            files=files,
        )



import re
from functools import partial

from mkdocs.config.defaults import MkDocsConfig
from mkdocs.contrib.search import SearchPlugin as BaseSearchPlugin
from mkdocs.structure.files import Files
from mkdocs.structure.pages import Page

from shadcn.filters import (
    active_section,
    file_exists,
    first_page,
    iconify,
    is_http_url,
    parse_author,
    setattribute,
)
from shadcn.plugins.mixins.dev import DevServerMixin
from shadcn.plugins.mixins.git import GitTimestampsMixin
from shadcn.plugins.mixins.mkdocstrings import MkdocstringsMixin
from shadcn.plugins.mixins.order import OrderMixin
from shadcn.plugins.mixins.table import TableMixin
from shadcn.plugins.mixins.markdown import MarkdownMixin


class SearchPlugin(
    GitTimestampsMixin,
    DevServerMixin,
    OrderMixin,
    MkdocstringsMixin,
    TableMixin,
    MarkdownMixin,
    BaseSearchPlugin,
):
    """⚠️ HACK ⚠️
    Custom plugin. As search is loaded by default, we subclass it so as
    to inject what we want (and without adding a list of additional plugins)
    """

    def on_config(self, config: MkDocsConfig):
        # we need to put "en" as default language for search
        self.config["lang"] = self.config.get("lang", None) or ["en"]
        return super().on_config(config)

    def on_env(self, env, /, *, config: MkDocsConfig, files: Files):
        # custom jinja2 filter
        env.filters["setattribute"] = setattribute
        env.filters["iconify"] = iconify
        env.filters["parse_author"] = parse_author
        env.filters["active_section"] = active_section
        env.filters["first_page"] = first_page
        env.filters["file_exists"] = partial(file_exists, config=config)
        env.filters["is_http_url"] = is_http_url
        return super().on_env(env, config=config, files=files)

    def on_page_markdown(
        self,
        markdown: str,
        /,
        *,
        page: Page,
        config: MkDocsConfig,
        files: Files,
    ):
        # remove first plain h1 if provided
        markdown = re.sub(r"^#\s+(.+)", r"", markdown, count=1)
        return super().on_page_markdown(
            markdown,
            page=page,
            config=config,
            files=files,
        )



from mkdocs.config.defaults import MkDocsConfig

from shadcn.plugins.mixins.base import Mixin

MKDOCSTRINGS_CONFIG = {
    "handlers": {
        "python": {
            "options": {
                "show_root_heading": True,
            }
        },
    },
    "default_handler": "python",
}


class MkdocstringsMixin(Mixin):
    def on_config(self, config: MkDocsConfig):
        plugin = config["plugins"].get("mkdocstrings", None)

        if plugin:
            options = (
                plugin.config.get("handlers", {})
                .get("python", {})
                .get("options", {})
            )
            show_root_heading = options.get("show_root_heading", None)
            if show_root_heading is None:
                plugin.config.update(MKDOCSTRINGS_CONFIG)

        return super().on_config(config)



from functools import wraps
from typing import Callable, List, Union

from bottle import Bottle  # type: ignore
from mkdocs.livereload import LiveReloadServer


class RouterMixin:
    """Mixin to add custom routes to the mkdocs dev server."""

    def __init__(self):
        # we use bottle to handle our routes
        # its .wsgi() method is very convenient with regards to
        # the _serve_request method of the mkdocs dev server
        self.bottle = Bottle()

    def add_route(
        self,
        path: str,
        handler: Callable,
        method: Union[str, List[str]] = "GET",
    ):
        """Add a route to the router."""
        self.bottle.route(path, method=method)(handler)  # type: ignore

    def extend_server(self, server: LiveReloadServer):
        """Extend the mkdocs dev server to add custom behavior."""
        original = server._serve_request

        @wraps(original)
        def _serve_request(environ, start_response):
            # put priority to base routes
            result = original(environ, start_response)
            if result is not None:
                return result
            # run our extra route handler
            return self.bottle.wsgi(environ, start_response)

        # monkey patch the _serve_request method of the server
        setattr(server, "_serve_request", _serve_request)

    def on_serve(self, server: LiveReloadServer, /, *, config, builder):
        """This method is called when the server is started. At the end of the mkdocs
        workflow. At this moment we have access to the live server to inject our new
        routes.
        See https://www.mkdocs.org/dev-guide/plugins/#events to see the mkdocs workflow.
        """
        self.extend_server(server)



import os
import re
import xml.etree.ElementTree as etree

from markdown.extensions import Extension
from markdown.inlinepatterns import InlineProcessor

CUSTOM_IMG_PATTERN = (
    r"""~[{]([^\]]*)[}][(]\s*(<)?([^()<>]+)(>)?(?:\s+(["'])(.*?)\5)?\s*[)]"""
)


class InlineExcalidrawProcessor(InlineProcessor):
    def __init__(
        self,
        pattern,
        md,
        svg_only: bool,
        base_dir: str,
    ):
        self.svg_only = svg_only
        self.base_dir = base_dir
        super().__init__(pattern, md)

    def handleMatch(self, m, data):
        title = m.group(1)
        href = m.group(3)
        desc = m.group(6)

        if self.svg_only:
            with open(
                os.path.join(self.base_dir, href.replace(".json", ".svg")),
                "r",
            ) as f:
                # remove namespace
                z = re.sub(r'\sxmlns="[^"]+"', "", f.read())
                svg = etree.fromstring(z)

            svg.set("xmlns", "http://www.w3.org/2000/svg")

            svg_desc = etree.Element("desc")
            svg_desc.text = desc
            svg.insert(0, svg_desc)

            svg_title = etree.Element("title")
            svg_title.text = title
            svg.insert(0, svg_title)

            return svg, m.start(0), m.end(0)

        html_id = os.path.basename(href)

        wrapper = etree.Element("div")
        wrapper.set("style", "width: 100%")

        div = etree.Element("div")
        div.set("id", html_id)
        div.set("data-scene", href)

        wrapper.append(div)

        el = etree.Element("script")
        el.set("type", "module")
        el.text = f"""window.excalidraw("{html_id}");"""

        wrapper.append(el)

        return wrapper, m.start(0), m.end(0)


class ExcalidrawExtension(Extension):
    # def __init__(self, svg_only: bool = False, base_dir: str = "excalidraw", **kwargs):
    #     self.svg_only = svg_only
    #     self.base_dir = base_dir
    #     super().__init__(**kwargs)
    def __init__(self, **kwargs):
        self.config = {
            "svg_only": [False, "Only load SVG files (build time)"],
            "base_dir": ["excalidraw", "Base directory for excalidraw files"],
        }
        super().__init__(**kwargs)

    def extendMarkdown(self, md):
        md.inlinePatterns.register(
            InlineExcalidrawProcessor(
                CUSTOM_IMG_PATTERN,
                md,
                base_dir=self.getConfig("base_dir"),
                svg_only=self.getConfig("svg_only"),
            ),
            "excalidraw",
            200,
        )


# Function to load the extension
def makeExtension(**kwargs):
    return ExcalidrawExtension(**kwargs)



from markdown.extensions import Extension
from markdown.inlinepatterns import InlineProcessor

from shadcn.filters import iconify

ICONIFY_TAG_RE = (
    r"[+]([a-z0-9\-]+:[a-z0-9\-]+)(?:[;][a-z]+[=][0-9a-zA-Z.#%]+)*[+]"
)


class IconifyInlinePattern(InlineProcessor):
    def handleMatch(self, m, data):
        raw_query = m.group(0).replace(m.group(1), "").strip("+").strip(";")
        if raw_query:
            params = dict(k.split("=") for k in raw_query.split(";"))
        else:
            params = {}

        icon_id = m.group(1).strip("+")
        raw_svg = iconify(icon_id, **params)
        raw_svg = raw_svg.replace(r"<svg", r'<svg class="iconify"')
        placeholder = self.md.htmlStash.store(raw_svg)
        return placeholder, m.start(0), m.end(0)


class IconifyExtension(Extension):
    def extendMarkdown(self, md):
        ICONIFY_PATTERN = IconifyInlinePattern(ICONIFY_TAG_RE, md)
        md.inlinePatterns.register(ICONIFY_PATTERN, "iconify", 175)


def makeExtension(**kwargs):
    return IconifyExtension(**kwargs)



from mkdocs.plugins import get_plugin_logger

from shadcn.plugins.mixins.base import Mixin

logger = get_plugin_logger("mixins/table")


class TableMixin(Mixin):
    """A mixin to wrap <table> to better manage overflow"""

    def on_page_content(self, html, **kwargs):
        return html.replace(
            r"<table",
            r'<div class="table-wrapper"><table',
        ).replace(r"</table>", r"</table></div>")
